package ui.view;

import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.AWTEventListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.text.DecimalFormat;
import java.util.Locale;
import javax.swing.JDialog;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.plaf.synth.SynthLookAndFeel;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import ui.model.Finisher;
import ui.model.Model;
import ui.model.ModelState;
import ui.model.OutputTray;
import ui.model.Tray.TrayNo;
import ui.model.framework.UIResourceNotFoundException;
import ui.model.framework.UIVersionMismatchException;
import ui.util.LoggingInitializer;
import ui.util.UncaughtExceptionHandler;
import ui.util.WriteStackTraceThreadGroup;
import ui.view.framework.memorymonitor.MemoryMonitor;
import ui.view.framework.multicast.IViewListener;
import ui.view.framework.multicast.ViewEvent;
import ui.view.product.MainWindow;
import ui.view.product.SplashWindow;
import ui.view.product.core.configuration.ConfigurationToolKit;
import ui.view.product.core.modellistener.AlignmentFileManagerListenerImpl;
import ui.view.product.core.modellistener.ConnectionListenerImpl;
import ui.view.product.core.modellistener.CustomPaperColorFileManagerListenerImpl;
import ui.view.product.core.modellistener.DFinFileManagerListenerImpl;
import ui.view.product.core.modellistener.IOTStateListenerImpl;
import ui.view.product.core.modellistener.ModelListenerImpl;
import ui.view.product.core.modellistener.OutputTrayListenerImpl;
import ui.view.product.core.modellistener.PaperTypeFileManagerListenerImpl;
import ui.view.product.core.modellistener.SYSStateListenerImpl;
import ui.view.product.core.modellistener.StockFileManagerListenerImpl;
import ui.view.product.core.modellistener.TrayListenerImpl;
import ui.view.product.core.multilanguage.components.MDialog;
import ui.view.product.core.multilanguage.resourceloader.ResourceManager;
import ui.view.product.core.powersaver.UserInputManager;
import ui.view.product.core.systemdialog.RuntimeExceptionDialog;
import ui.view.product.core.timer.AutoCloseTrayPropertiesTimer;
import ui.view.product.core.timer.AutoLogoffTimer;
import ui.view.product.core.timer.AutoPauseTimer;
import ui.view.product.core.timer.CEModeOffTimer;
import ui.view.product.core.translationlayer.ModelStateEventManager;
import ui.view.product.core.translationlayer.UIStatusKit;
import ui.view.product.core.utils.SingletonWindowInitializer;
import ui.view.product.popup.PopupInitilaizer;
import ui.view.product.popup.VersionErrorPopup;

public final class ProductLauncher implements Runnable {

   private static final Log log = LogFactory.getLog(ProductLauncher.class);
   private static final String PRODUCT_UI_VERSION = "V8.0 SynthLookAndFeel And Java 1.6.0 ";
   private static final String THREAD_GROUP_NAME = "AWT-EventQueue";
   private static ThreadGroup writeStackTraceThreadGroup = null;


   public static void main(String[] args) {
      Locale locale = Locale.getDefault();
      Locale.setDefault(Locale.US);
      System.setProperty("sun.java2d.noddraw", "true");

      try {
         LoggingInitializer.initialize();
         displayLocaleLog(locale);
         log.info("Product UI Version V8.0 SynthLookAndFeel And Java 1.6.0 ");
         log.info("PATH = " + System.getenv("PATH"));
         log.info("java.library.path = " + System.getProperty("java.library.path"));
         (new Thread(getThreadGroup(), MemoryMonitor.getInstance())).start();
         (new Thread(getThreadGroup(), new ProductLauncher())).start();
      } catch (Exception var3) {
         throw new UIResourceNotFoundException(var3);
      }
   }

   private static void displayLocaleLog(Locale locale) {
      log.debug("Set Default Locale " + Locale.US);
      log.debug("Before Locale Properties ");
      displayLocaleProperties(locale);
      log.debug("System Properties: " + System.getProperties());
      log.debug("After Locale Properties ");
      displayLocaleProperties(Locale.getDefault());
   }

   private static void displayLocaleProperties(Locale locale) {
      log.debug("JVM Default Locale -  getDefault() :" + locale);
      String[] countryCode = Locale.getISOCountries();
      log.debug("Locale ISO 639 Code - getISOCountries() :" + countryCode);
      log.debug("Locale Name -  getDisplayName() :" + locale.getDisplayName());
      log.debug("Locale Country Code -  getCountry() :" + locale.getCountry());
      log.debug("Locale Country Name - getDisplayCountry() :" + locale.getDisplayCountry());
      log.debug("Locale Language - getDisplayLanguage() :" + locale.getDisplayLanguage());
      log.debug("Locale Language Code - getLanguage() :" + locale.getLanguage());
      log.debug("Locale Varient - getDisplayVariant() :" + locale.getDisplayVariant());
      log.debug("Locale Variant Code - getVariant() :" + locale.getVariant());
      log.debug("Locale Country - getISO3Country() :" + locale.getISO3Country());
      log.debug("Locale Name - toString() :" + locale.toString());
   }

   public static void terminate(int exitCode) {
      log.debug("Product Launcher terminate : " + exitCode);
      Model.getInstance().shutdown();
      System.exit(exitCode);
   }

   private static void asignListeners() {
      log.debug("Asign Listeners");
      Model.getInstance().addModelEventListener(new ModelListenerImpl());
      Model.getInstance().getIOT().addIOTStateListener(new IOTStateListenerImpl());
      Model.getInstance().getSYS().addSYSStateListener(new SYSStateListenerImpl());
      Model.getInstance().getCustomPaperColorFileManager().addFileManagerListener(new CustomPaperColorFileManagerListenerImpl());
      Model.getInstance().getPaperTypeFileManager().addFileManagerListener(new PaperTypeFileManagerListenerImpl());
      Model.getInstance().getStockFileManager().addFileManagerListener(new StockFileManagerListenerImpl());
      Model.getInstance().getAlignmentFileManager().addFileManagerListener(new AlignmentFileManagerListenerImpl());
      Model.getInstance().getDFinFoldFileManager().addFileManagerListener(new DFinFileManagerListenerImpl());
      Model.getInstance().getIOT().addTrayListener(TrayNo.TRAY1, new TrayListenerImpl(TrayNo.TRAY1));
      Model.getInstance().getIOT().addTrayListener(TrayNo.TRAY2, new TrayListenerImpl(TrayNo.TRAY2));
      Model.getInstance().getIOT().addTrayListener(TrayNo.TRAY3, new TrayListenerImpl(TrayNo.TRAY3));
      Model.getInstance().getIOT().addTrayListener(TrayNo.HCF1_1, new TrayListenerImpl(TrayNo.HCF1_1));
      Model.getInstance().getIOT().addTrayListener(TrayNo.HCF1_2, new TrayListenerImpl(TrayNo.HCF1_2));
      Model.getInstance().getIOT().addTrayListener(TrayNo.HCF2_1, new TrayListenerImpl(TrayNo.HCF2_1));
      Model.getInstance().getIOT().addTrayListener(TrayNo.HCF2_2, new TrayListenerImpl(TrayNo.HCF2_2));
      Model.getInstance().getIOT().addTrayListener(TrayNo.MSI, new TrayListenerImpl(TrayNo.MSI));
      Model.getInstance().getIOT().addTrayListener(TrayNo.DFIN_INTERPOSER, new TrayListenerImpl(TrayNo.DFIN_INTERPOSER));
      Model.getInstance().getConnection().addConnectionListener(new ConnectionListenerImpl());
      ModelStateEventManager.addChangedReadyListener(new IViewListener() {
         public void viewEventFired(ViewEvent e) {
            ProductLauncher.assignOutputTrayListener();
         }
      });
      ModelStateEventManager.addStartedListener(new IViewListener() {
         public void viewEventFired(ViewEvent event) {
            MainWindow.getInstance().setVisibility();
            MainWindow.getInstance().setVisible(true);
            ModelState state = UIStatusKit.getInstance().getUIState();
            ProductLauncher.log.debug(state);
         }
      });
      ModelStateEventManager.addStartedListener(new IViewListener() {
         public void viewEventFired(ViewEvent event) {
            SplashWindow.getInstance().setVisible(false);
         }
      });
      UIStatusKit.getInstance().addViewStateChangeListener(new IViewListener() {
         public void viewEventFired(ViewEvent event) {
            ModelState state = UIStatusKit.getInstance().getUIState();
            if(state == ModelState.CONNECTING) {
               MainWindow.getInstance().disposeAllChildWindows();
            }

         }
      });
      Toolkit.getDefaultToolkit().addAWTEventListener(new AWTEventListener() {
         public void eventDispatched(AWTEvent event) {
            Window current = MDialog.getCurrentWindow();
            if(event.getSource() instanceof Component) {
               Window parent = ProductLauncher.getParentDialog((Component)event.getSource());
               if(!(parent instanceof MDialog) && parent instanceof JDialog) {
                  return;
               }

               if(parent != current) {
                  if(event instanceof KeyEvent) {
                     ((KeyEvent)event).consume();
                  } else if(event instanceof MouseEvent) {
                     ((MouseEvent)event).consume();
                  }
               }
            }

         }
      }, 24L);
      PopupInitilaizer.initializeListener();
      SingletonWindowInitializer.initializeListener();
   }

   private static Window getParentDialog(Component compnent) {
      while(true) {
         if(((Component)compnent).getParent() != null) {
            compnent = ((Component)compnent).getParent();
            if(compnent instanceof MDialog) {
               return (Window)compnent;
            }

            if(compnent instanceof MainWindow) {
               return (Window)compnent;
            }

            if(!(compnent instanceof JDialog)) {
               continue;
            }

            return (Window)compnent;
         }

         return null;
      }
   }

   private static ThreadGroup createThreadGroup() {
      WriteStackTraceThreadGroup group = new WriteStackTraceThreadGroup("AWT-EventQueue");
      group.addUncaughtExceptionHandler(new UncaughtExceptionHandler() {
         public void uncaughtException(Thread t, Throwable e) {
            ProductLauncher.log.error(t, e);
            if(!ProductLauncher.isSVGError(e)) {
               if(e instanceof UIVersionMismatchException) {
                  SwingUtilities.invokeLater(new Runnable() {
                     public void run() {
                        VersionErrorPopup.getInstance().setVisible(true);
                     }
                  });
               } else {
                  RuntimeExceptionDialog.openFrame(e);
               }
            }
         }
      });
      return group;
   }

   private static boolean isSVGError(Throwable e) {
      return e.toString().equals("java.util.ConcurrentModificationException");
   }

   private static ThreadGroup getThreadGroup() {
      if(writeStackTraceThreadGroup == null) {
         writeStackTraceThreadGroup = createThreadGroup();
      }

      return writeStackTraceThreadGroup;
   }

   public static void setLookAndFeel() {
      log.debug("Setting Look And Feel");
      SynthLookAndFeel synth = new SynthLookAndFeel();
      Class aClass = ProductLauncher.class;
      BufferedInputStream xFile = null;

      try {
         xFile = new BufferedInputStream(new FileInputStream(ConfigurationToolKit.getInstance().getResourcePath().getImageDir() + "xerox-synth.xml"));
         synth.load(xFile, aClass);
         UIManager.setLookAndFeel(synth);
      } catch (Exception var4) {
         throw new UIResourceNotFoundException(var4);
      }
   }

   public static void setWindowsLookAndFeel() {
      try {
         UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      } catch (Exception var1) {
         throw new UIResourceNotFoundException(var1);
      }
   }

   private static void assignOutputTrayListener() {
      log.debug("Assign Output Tray Listener");
      Finisher finisher = Model.getInstance().getIOT().getFirstFinisher();
      OutputTray[] outputTray = finisher.getOutputTrays();

      for(int outputTray2 = 0; outputTray2 < outputTray.length; ++outputTray2) {
         outputTray[outputTray2].addOutputTrayListener(new OutputTrayListenerImpl());
      }

      while(finisher.hasNextFinisher()) {
         finisher = finisher.nextFinisher();
         OutputTray[] var4 = finisher.getOutputTrays();

         for(int i = 0; i < var4.length; ++i) {
            var4[i].addOutputTrayListener(new OutputTrayListenerImpl());
         }
      }

   }

   private static void initializeTimers() {
      UserInputManager.getInstance().startMonitoring();
      AutoLogoffTimer.getInstance();
      CEModeOffTimer.getInstance();
      AutoCloseTrayPropertiesTimer.getInstance();
      AutoPauseTimer.getInstance();
   }

   public void run() {
      if(Model.getInstance().existInstance()) {
         log.debug("UI already exist");
         System.exit(0);
      }

      log.debug("started!");
      log.debug("###MEMORY[FreeMemory / TotalMemory / MaxMemory](byte)###");
      log.debug((new DecimalFormat("###,###")).format(Runtime.getRuntime().freeMemory()) + " / " + (new DecimalFormat("###,###")).format(Runtime.getRuntime().totalMemory()) + " / " + (new DecimalFormat("###,###")).format(Runtime.getRuntime().maxMemory()));
      setLookAndFeel();
      ResourceManager.getInstance();
      asignListeners();
      initializeTimers();
      SwingUtilities.invokeLater(new Runnable() {
         public void run() {
            MemoryMonitor.getInstance().addMonitorThread(Thread.currentThread());
         }
      });
      (new Thread(Thread.currentThread().getThreadGroup(), new Runnable() {
         public void run() {
            Model.getInstance().start();
         }
      })).start();
      SplashWindow.getInstance().setVisible(true);
      MainWindow.getInstance();
      Model.getInstance().notifyViewReady();
   }

}
